/*
  Copyright 2019 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.auth

import com.kakao.sdk.auth.model.OAuthToken

/**
 * 카카오 API 에 사용되는 액세스 토큰, 리프레시 토큰을 관리하는 저장소.
 */
interface TokenManageable {
    /**
     * 저장되어 있는 [OAuthToken] 반환.
     */
    fun getToken(): OAuthToken?

    /**
     * 토큰을 저장.
     *
     * @param token 저장하고자 하는 [OAuthToken] 객체.
     */
    fun setToken(token: OAuthToken)

    /**
     * 저장되어 있는 [OAuthToken] 객체를 삭제.
     */
    fun clear()
}

object TokenManagerProvider {
    var currentManager: TokenManageable = DefaultTokenManager()
}
